package ru.dbotthepony.mc.otm.capability;

import net.minecraftforge.common.capabilities.*;
import net.minecraftforge.energy.IEnergyStorage;
import ru.dbotthepony.mc.otm.capability.android.IAndroidCapability;
import ru.dbotthepony.mc.otm.capability.drive.IMatteryDrive;
import ru.dbotthepony.mc.otm.capability.matter.IMatterGridCell;
import ru.dbotthepony.mc.otm.capability.matter.IMatterHandler;
import ru.dbotthepony.mc.otm.capability.matter.IMatterTaskProvider;
import ru.dbotthepony.mc.otm.capability.matter.IPatternStorage;

import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;

public class MatteryCapability {
	public static final Capability<IMatteryEnergyStorage> ENERGY = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IAndroidCapability> ANDROID = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IMatterHandler> MATTER = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IMatterGridCell> MATTER_CELL = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IPatternStorage> PATTERN = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IMatterTaskProvider> TASK = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IMatteryDrive> DRIVE = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IStorageGridCell> STORAGE_CELL = CapabilityManager.get(new CapabilityToken<>() {});

	@SuppressWarnings("unused")
	public static void register(final RegisterCapabilitiesEvent event) {
		event.register(IMatteryEnergyStorage.class);
		event.register(IAndroidCapability.class);
		event.register(IMatterHandler.class);
		event.register(IMatterGridCell.class);
		event.register(IPatternStorage.class);
		event.register(IMatterTaskProvider.class);
		event.register(IMatteryDrive.class);
		event.register(IStorageGridCell.class);
	}

	public static final MathContext ROUND_RULES = new MathContext(32, RoundingMode.HALF_DOWN);

	public static final BigDecimal INT_MAX_VALUE = new BigDecimal(Integer.MAX_VALUE);
	public static final BigDecimal INT_MIN_VALUE = new BigDecimal(Integer.MIN_VALUE);

	public static final BigDecimal LONG_MAX_VALUE = new BigDecimal(Long.MAX_VALUE);
	public static final BigDecimal LONG_MIN_VALUE = new BigDecimal(Long.MIN_VALUE);

	public static BigDecimal drainFE(IEnergyStorage capability, BigDecimal howMuch, boolean simulate) {
		if (howMuch.compareTo(BigDecimal.ONE) < 0)
			return BigDecimal.ZERO;

		return new BigDecimal(capability.extractEnergy(howMuch.compareTo(INT_MAX_VALUE) > 0 ? Integer.MAX_VALUE : howMuch.intValue(), simulate));
	}

	public static BigDecimal floodFE(IEnergyStorage capability, BigDecimal howMuch, boolean simulate) {
		if (howMuch.compareTo(BigDecimal.ONE) < 0)
			return BigDecimal.ZERO;

		return new BigDecimal(capability.receiveEnergy(howMuch.compareTo(INT_MAX_VALUE) > 0 ? Integer.MAX_VALUE : howMuch.intValue(), simulate));
	}
}
